package com.huimv.eartag2.eartag.listener;

import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.mq.Const;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.MathUtil;
import com.huimv.eartag2.eartag.service.IDeviceService;
import com.huimv.eartag2.eartag.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class EartagListener {
    @Autowired
    private IDeviceService deviceService;
    @Autowired
    private IEartagService eartagService;

    /**
     * @Method : processRawdata
     * @Description :
     * @Params : [RawMap]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/3/12
     * @Time : 17:53
     */
    @RabbitListener(queues = Const.QUEUE_ASK_EARTAG)
    @RabbitHandler
    public void processRawdata(Map RawMap) throws ParseException, IOException {
        System.out.println("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 处理耳标 <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
        String askText = RawMap.get("askText").toString();

        //本模块主要执行保存以下4类数据：（其他注册数据，在线数据，状态数据等都是由其他模块工程执行处理-process2模块工程）
        //--保存原始流水数据（原始流水表）
        //--保存设备心跳流水数据（设备心跳流水表）
        //--保存设备环境流水数据（设备温度流水表）
        //--保存耳标流水数据（耳标流水表）

        //测试-临时写入耳标数据到文件
        DateUtil dateUtil = new DateUtil();
//        if(askText.trim().indexOf("330110004000010") != -1){
//            writeTxt(dateUtil.getTodayMissionText()+" "+askText,"eartag");
//        }
        //写入所有接收到的耳标数据
//        writeTxt(dateUtil.getTodayMissionText()+" "+askText,"all_eartag");

        //{处理请求报文}
        Map askMap = handleAskText(askText);
        if (askMap == null){
            return;
        }
        String type = askMap.get("type").toString();
        JSONObject dataJo = (JSONObject) askMap.get("data");
        System.out.println("######## eartagJo="+dataJo);
        if (type.trim().equalsIgnoreCase("eartag") /*&&  getAskTime(dataJo.getString("askTime"))*/) {
            //{处理耳标数据}
            eartagService.handleEartag(dataJo);
            //调用省平台的接口，将耳标数据发送到省平台
//            eartagService.sendSowProvincePlatform(dataJo,askText);
        }

    }
    public boolean getAskTime(String askTime)   {
        String newAskTime = askTime.substring(0, 4) + "-" + askTime.substring(4, 6) + "-" + askTime.substring(6, 8) + " " + askTime.substring(8, 10) + ":" + askTime.substring(10, 12) + ":" + askTime.substring(12, 14);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date targetDate = sdf.parse(newAskTime);
            Date currentDate = new Date();

            // 获取时间戳
            long targetTimestamp = targetDate.getTime();
            long currentTimestamp = currentDate.getTime();

            // 设置时间差阈值，例如10分钟
            long maxTimeDifference = 60 * 1000; // 10 minutes in milliseconds

            // 计算时间差
            long timeDifference = Math.abs(targetTimestamp - currentTimestamp);

            if (timeDifference <= maxTimeDifference) {
                return true;
            } else {
                return false;
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return false;
    }

    public void writeTxt(String conent,String file) throws ParseException, IOException {
        String todayText = getTodayDateText();
        File f = new File("/opt/log/log_"+file+"_"+todayText+".txt");
        if (f.exists()) {
        } else {
            f.createNewFile();// 不存在则创建
        }
        BufferedWriter output = new BufferedWriter(new FileWriter(f,true));//true,则追加写入text文本
        output.write(conent);
        output.write("\r\n");//换行
        output.flush();
        output.close();
    }

    public String getTodayDateText() throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(new Date());
    }

    private boolean checkTestRange(String earmark) {
        Long c = Long.parseLong(earmark);
        Long a = 122083123610001L;
        Long b = 122083123610100L;
        return (c >= a && c < b) || (c > a && c <= b);
    }

    /**
     * @Method : handleAskText
     * @Description : 处理请求
     * @Params : [askText]
     * @Return : java.util.Map
     * @Author : ZhuoNing
     * @Date : 2022/2/9
     * @Time : 13:38
     */
    private Map handleAskText(String askText) {
        Map outMap = new HashMap();
        JSONObject dataJo = new JSONObject();
        String type = "";
        String[] dataArray = askText.split("\\+");
        String cmdHeader = dataArray[0];
        if (cmdHeader.trim().equalsIgnoreCase("hm")) {
            //采集器原始数据
            String device = dataArray[1];
            String cmd = dataArray[2];
            if (cmd.trim().equalsIgnoreCase("0")) {
                //不需要处理
            } else if (cmd.trim().equalsIgnoreCase("1")) {
                //获取设备编码
            } else if (cmd.trim().equalsIgnoreCase("2")) {
                //获取服务器时间命令
            } else if (cmd.trim().equalsIgnoreCase("3")) {
                //心跳包命令
                dataJo.put("device", device);
                type = "device-heartbeat";
            } else if (cmd.trim().equalsIgnoreCase("4")) {
                //环境温度-设备、数值、上传时间
                dataJo.put("device", dataArray[1]);
                dataJo.put("temp", dataArray[4]);
                dataJo.put("askTime", dataArray[5]);
                type = "device-temp";
            } else {
                log.error("##当前数据为非法数据>>" + askText);
            }
        } else if (cmdHeader.trim().equalsIgnoreCase("zj")) {
//            String device = dataArray[1];
            //-- 耳标数据 --//
            //计算耳标温度
            Double earTemp = 0D;
            Double earTemp2 = 0D;
            String other = dataArray[9];
            if (other != null && other.trim().length() > 0) {
                String[] otherArray = other.split("#");
                String resi = otherArray[2];
                if (otherArray.length>3){
                    String resi2 = otherArray[3];
                    if (null != resi2 && !"0".equals(resi2)){

                        Double temp2 = MathUtil.ln( Integer.parseInt(resi2));
                        earTemp2 = new BigDecimal(temp2).setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                    }
                }

                int resiInt = Integer.parseInt(resi);
                if (resiInt != 0) {
                    Double temp = MathUtil.ln(resiInt);
                    earTemp = new BigDecimal(temp).setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                }

            }
            //计算运动量//
            Integer act1Int = eartagService.countAct(dataArray[2], dataArray[6]);
            //命令头
            dataJo.put("cmdHeader", dataArray[0]);
            //采集器id
            dataJo.put("device", dataArray[1]);
            //耳标号
            dataJo.put("earmark", dataArray[2]);
            //电池电量
            dataJo.put("bat", dataArray[3]);
            //耳标温度
            dataJo.put("earTemp", dataArray[4]);
            if (earTemp<2 ||earTemp >40){
                System.out.println("温度数据有误,使用上一包数据");
                return null;
            }
            dataJo.put("earTemp1", earTemp);
            dataJo.put("earTemp2", earTemp2);
            //环境温度
            dataJo.put("envTemp", dataArray[5]);
            dataJo.put("envTemp1", MathUtil.countEnvtemp(dataArray[5], 2));
            //运动量
            dataJo.put("act", dataArray[6]);
            dataJo.put("act1", act1Int);
            //信号强度
            dataJo.put("signal", dataArray[7]);
            //采集时间
            dataJo.put("askTime", dataArray[8]);
            //预留字段
            dataJo.put("other", dataArray[9]);
            type = "eartag";
        }
        outMap.put("type", type);
        outMap.put("data", dataJo);
        return outMap;
    }
}
